# -*- coding: utf-8 -*-

import os, sys, zmq, random, pprint, logging
import operator, pickle, asyncio, threading
from collections import deque, Counter, OrderedDict
from PyQt5 import QtGui, QtCore, QtWidgets, QtMultimedia
from PyQt5.QtCore import Qt, pyqtSlot as Slot, pyqtSignal as Signal, pyqtProperty as Property
from PyQt5.QtOpenGL import QGL, QGLFormat, QGLWidget
from robot import Server, AllMoves
try:
    import resource_qrc
except ImportError:pass
    

__author__ = "Obsession <2608135678@qq.com>"
log = lambda *j:logging.getLogger(__name__).warning(
        '   '.join([pprint.pformat(i, width=150) for i in j]))


class Client(QtCore.QThread):
    dataOut = Signal(object)
    def __init__(self, parent=None):
        super().__init__(parent)
        self.__socket = None
        self.__data = None
        self.__running = True
        self.__interval = 999999
        self.event = threading.Event()

    def send(self, data):
        if self.__socket is not None:
            self.__data = data
            self.event.set()                 # 释放阻塞

    async def handle(self):
        task = await self.dumps(self.__data) # 转为二进制
        self.__socket.send(task)             # 发送命令
        if self.__data == 'stop':            # 让服务器停止
            self.__running = False           # 自己也退出
            return
        message = self.__socket.recv()       # 获取回复
        result = await self.loads(message)   # 二进制转数据
        self.dataOut.emit(result)            # 打开绑定函数
        self.__data = None

    async def loads(self, data):
        return pickle.loads(data)

    async def dumps(self, data):
        return pickle.dumps(data)

    async def EventPool(self):
        self.event.clear()                 # 开启阻塞
        while self.__running:
            try:
                self.event.wait(self.__interval)
                if self.__data is None:
                    pass
                elif self.event.is_set():  # 无阻塞通行
                    await self.handle()
                self.event.clear()
            except Exception as exc:
                logging.exception(f"客户端异常: {exc}", exc_info=True)
                break
        print('客户端退出 ...')

    def run(self):
        context = zmq.Context()
        self.__socket = context.socket(zmq.REQ)
        self.__socket.connect("tcp://localhost:6789")
        asyncio.run(self.EventPool())


class Handy(QtCore.QObject):
    def __init__(self):
        super().__init__()
        self.master = None
        self.cache_dir = Settings.cache_dir()
        self.current_dir = os.sep.join(__file__.split(os.sep)[:-1])
        self.resource_dir = f'{self.current_dir}/resource'
        self.config_dir = f'{self.resource_dir}/config'
        self.getColor = lambda i=0:QtGui.QColor(*(QtCore.qrand()%256 for j in 'rgb'))
        self.head_pos = [(i,j) for i in range(25) for j in range(4)]
        self.config = self.createConfig()

    def createConfig(self):
        default = {'curve': 0, 'my_head':[24,3], 'petyle': 1,
                   'board_offset0':[23, 13, 110, 112],
                   'board_offset1':[5, 10, 78, 79],
                   'board_offset2':[5, 5, 76, 76], }
        setting = Settings(f'{self.config_dir}/settings.ini')
        for k,v in default.items():
            if not setting.has(k):
                setting.set(k,v)
        return setting

    def schedule(self, target, *args, delay=0, **kwargs):
        """ 延迟执行事件 timer = QtCore.QTimer(self, singleShot=True) """
        QtCore.QTimer.singleShot(delay, lambda:target(*args, **kwargs))

    def getFontWidht(self, string="中", font=None, spacing=None, limit=100):
        """ 获取字体宽度, limit 限制文本宽度 """
        if font is None:
            font = QtWidgets.QApplication.font()
        metrics = QtGui.QFontMetrics(font)
        if not isinstance(spacing, int):
            spacing = metrics.leading()
        return metrics.horizontalAdvance(string) + len(string) * spacing
        metrics.elidedText(string, Qt.ElideRight, limit)

    def fillColor(self, background):
        """ 返回背景颜色上的字体颜色, 黑或白 """
        return (lambda r,g,b,a:1-(0.299*r+0.587*g+0.114*b)/255)(*background.getRgb())<0.5 and Qt.black or Qt.white

    def getScreen(self):
        """ 屏幕参数 """
        return QtGui.QGuiApplication.primaryScreen().availableGeometry()

    def centerScreen(self, window, size=None):
        """ 设置窗口屏幕居中 """
        if size is not None:
            window.resize(size)
        original = window.frameGeometry()
        original.moveCenter(self.getScreen().center())
        window.move(original.topLeft())

    def twinkle(self, widget, color='ff0000', value=0, duration=250, loop=1):
        """ 闪烁 """
        anim = QtCore.QPropertyAnimation(widget, b'color', self)
        anim.setStartValue(QtGui.QColor(value))
        anim.setEndValue(QtGui.QColor(value))
        anim.setKeyValueAt(0.1, QtGui.QColor(color))
        anim.setDuration(duration)
        anim.setLoopCount(loop)
        anim.start()

    def createMusic(self, fname, volume=100):
        """ 播放音乐 mp3 格式文件 """
        fp = os.path.join(self.resource_dir, 'sound', fname)
        fname = fp if os.path.isfile(fp) else f':/{fname}'
        content = QtMultimedia.QMediaContent(QtCore.QUrl.fromLocalFile(fname))
        player = QtMultimedia.QMediaPlayer(self)
        player.setMedia(content)
        player.setVolume(volume)
        return player

    def readPixmap(self, fname, mode='pixmap'):
        """ Image 模式用于读取大图片, 节省内存 """
        for fd in [':', self.resource_dir]:
            if mode == 'image':
                image = QtGui.QImage(f'{fd}/{fname}')
                image = QtGui.QPixmap.fromImage(image)
            else:
                image = QtGui.QPixmap(f'{fd}/{fname}')
            if not [image.width(), image.height()].count(0):
                return image
        raise ValueError(f"读取图片失败: {self.resource_dir}/{fname}")


class Settings(QtCore.QObject):
    """ 用户设置 """
    QtCore.QCoreApplication.setOrganizationName("Temp")
    QtCore.QCoreApplication.setApplicationName("Obsesson")
    QtCore.QCoreApplication.setApplicationVersion("1.0")   # 设置缓存(目录/名称/版本)
    QtCore.QCoreApplication.setOrganizationDomain("Obsession <2608135678@qq.com>")

    @staticmethod
    def cache_dir(*args, version=None):
        """ 返回应用程序缓存目录 """
        location = QtCore.QStandardPaths.CacheLocation
        cachedir = QtCore.QStandardPaths.writableLocation(location)
        if version is not None:
            version = QtCore.QCoreApplication.applicationVersion()
            cachedir = os.path.join(cachedir, version)
        if not os.path.exists(cachedir):
            os.makedirs(cachedir)
        return os.path.abspath(os.path.join(cachedir, *args) if args else cachedir)

    def __init__(self, fpath=None):
        super().__init__()
        sfp = fpath if fpath else self.__class__.__name__
        mat = QtCore.QSettings.IniFormat if fpath else 'Cache'
        self.settings = QtCore.QSettings(sfp, mat)
        self.settings.setIniCodec('utf-8')
        self.settings.sync()

    def has(self, key):
        return self.settings.contains(key)

    def get(self, key, default=None):
        return self.settings.value(key) if self.has(key) else default

    def keys(self):
        return self.settings.allKeys()

    def set(self, key, value):
        self.settings.setValue(key, value)

    def remove(self, key):
        self.settings.remove(key)

    def clear(self):
        self.settings.clear()


class ParallelMachine(QtCore.QObject):
    """ 并行动画组, 最好开始把实例设为模块属性, 最后统一执行
        animationEffects 中涵盖了大多数动画效果 """
    started = Signal()
    finished = Signal()
    animationEffects = {i:getattr(QtCore.QEasingCurve, i)
                    for i in dir(QtCore.QEasingCurve) if i[:2] in ('In', 'Ou')}
    def __init__(self, batch=(), parent=None, curve=None, once=True) -> None:
        """ curve: 如果为整数,将从 animationEffects 动画列表中按索引选择,
            once: 为动画执行次数, 默认执行一次关闭状态机 """
        super().__init__(parent)
        self.curve = curve
        if isinstance(curve, int) and len(self.animationEffects) > curve:
            effects = list(self.animationEffects.keys())[curve]
            self.curve = self.animationEffects[effects]
        self.root_state = QtCore.QState()                                         # 定义一个根状态作为父状态
        self.start_state = QtCore.QState(self.root_state)                         # 开始状态
        self.end_state = QtCore.QState(self.root_state)                           # 结束状态
        self.root_state.setInitialState(self.start_state)                         # 根状态的初始状态
        self.par_group = QtCore.QParallelAnimationGroup()                         # 并行动画组
        self.seq_group = QtCore.QSequentialAnimationGroup()                       # 串行动画组
        if once:self.end_state.entered.connect(self.stop)                         # 绑定停止函数
        if batch:self.startBatch(batch)                                           # 批量执行模式

    def addAnimation(self, item, attr, start, end, duration=300, kvAt=()) -> None:
        """ 添加动画 """
        self.start_state.assignProperty(item, attr, start)                        # 添加开始状态属性值
        self.end_state.assignProperty(item, attr, end)                            # 添加结束状态属性值
        anim = QtCore.QPropertyAnimation(item, attr.encode('ascii'))              # 定义动画
        anim.setDuration(duration)                                                # 动画执行总时长
        for k,v in kvAt:anim.setKeyValueAt(k,v)                                   # 动画加入插值
        if self.curve is not None:
            anim.setEasingCurve(self.curve)                                       # 设置动画曲线效果
        self.par_group.addAnimation(anim)                                         # 动画添加到动画组
        # self.machine.addDefaultAnimation(anim)                                    # 设置默认动画

    def startAnimation(self) -> None:
        """ 交由状态机执行动画 """
        tran = self.root_state.addTransition(self.started, self.end_state)        # 添加转换方式
        tran.addAnimation(self.par_group)                                         # 添加动画组
        self.machine = QtCore.QStateMachine(self)                                 # 定义状态机
        self.machine.addState(self.root_state)                                    # 根状态添加到状态机
        self.machine.setInitialState(self.root_state)                             # 设置初始状态
        # configuration = self.machine.configuration()
        self.machine.start()
        QtCore.QTimer.singleShot(1, lambda:self.started.emit())

    def stop(self):
        self.end_state.entered.disconnect(self.stop)
        self.machine.stop()
        self.finished.emit()

    def startBatch(self, batch):
        """ [(item, 'pos', old_pos, new_pos, 1000)] """
        for x in batch:
            self.addAnimation(*x)
        self.startAnimation()


class LegendItem(QtWidgets.QGraphicsObject):
    def __init__(self, parent=None, **kwds):
        self.parent = parent
        self.__boundingRect = None
        self.__font = kwds.pop('font', QtGui.QFont("Arial", 9))
        self.__pen = kwds.pop('pen', QtGui.QPen(Qt.NoPen))
        self.__path = kwds.pop('path', QtGui.QPainterPath())
        self.__background = kwds.pop('background', QtGui.QColor())
        self.__info = kwds.pop('info', None)
        self.__picture = kwds.pop('picture', None)
        super().__init__(**kwds)
        self.setZValue(1e9)
        self.shadow = QtWidgets.QGraphicsDropShadowEffect()
        self.shadow.setOffset(QtCore.QPointF(10, 10))
        self.shadow.setColor(Qt.black)
        self.shadow.setBlurRadius(20)
        self.shadow.setEnabled(True)
        self.setGraphicsEffect(self.shadow)

        self.setAcceptHoverEvents(True)             # 可悬停
        self.setAcceptTouchEvents(True)             # 可触摸
        self.setFlag(self.ItemIsMovable, True)      # 可移动
        self.setFlag(self.ItemIsSelectable, True)   # 可选中
        if 'rect' in kwds:self.setRect(rect)

    def paint(self, painter, option, widget):
        painter.fillRect(self.__boundingRect, self.__background)
        if self.__info is not None:
            self.draw_info(painter)
        if self.picture is not None:
            painter.drawPicture(0, 0, self.picture)

    def boundingRect(self):
        if self.__boundingRect is None:
            br, pw = self.__path.controlPointRect(), self.__pen.widthF()
            self.__boundingRect = QtCore.QRectF(br.adjusted(-pw, -pw, pw, pw))
        return self.__boundingRect

    @Slot(QtCore.QRectF)
    def setRect(self, rect):
        self.prepareGeometryChange()
        self.__boundingRect = None
        self.__path = QtGui.QPainterPath()
        self.__path.addRect(QtCore.QRectF(rect))
        self.update()

    @Property(QtGui.QPicture)
    def picture(self):
        return self.__picture

    @picture.setter
    def picture(self, picture):
        if picture is None:
            del self.picture
        elif self.__picture != picture:
            self.setFlag(self.ItemIsMovable, False)
            self.setVisible(True)
            self.__picture = picture
            self.update()

    @picture.deleter
    def picture(self,):
        self.__picture = None
        self.setVisible(False)
        self.setFlag(self.ItemIsMovable, True)

    def draw_info(self, painter):
        painter.setFont(self.__font)
        painter.setBrush(Qt.NoBrush)
        x, y, w, h = self.__boundingRect.getRect()
        tw, th = 100, 25
        for i,(key,val) in enumerate(Touch.dd.items()):
            painter.save()
            painter.setPen(Qt.cyan)
            painter.drawText(QtCore.QRectF(10+ x,    10+ i*th, tw, th), Qt.AlignVCenter, key)
            painter.setPen(Qt.yellow)
            painter.drawText(QtCore.QRectF(10+ x+tw, 10+ i*th, w-tw, th), Qt.AlignVCenter, f'{val}')
            painter.restore()


class PixmapItem(QtWidgets.QGraphicsObject):
    checked = Signal(object, object)
    def __init__(self, pixmap, parent=None, **kwds):
        check = kwds.pop('check', None)
        self.__shadow = None
        self.__clip = kwds.pop('clip', None)
        if 'pos' in kwds and isinstance(kwds['pos'], (tuple, list)):
            kwds.update(dict(pos=QtCore.QPointF(*kwds['pos'])))
        if isinstance(pixmap, str):
            pixmap = handy.readPixmap(pixmap)
        super().__init__(parent, **kwds)
        self.pixmap, rect = pixmap, pixmap.rect()
        if self.__clip is not None:
            rect = QtCore.QRectF(rect.topLeft(), self.__clip.size())
        self.__boundingRect = QtCore.QRectF(rect)
        if check is not None:
            self.checked.connect(check)
            self.__shadow = self.setEffect()
        self.setAcceptTouchEvents(True)             # 可触摸
        self.setAcceptHoverEvents(True)             # 可悬停
        self.setFlag(self.ItemIsSelectable, True)   # 可选中
        self.setFlag(self.ItemIsFocusable, True)    # 有焦点
        self.setCacheMode(self.ItemCoordinateCache)

    def paint(self, painter, option, widget):
        painter.setRenderHint(QtGui.QPainter.Antialiasing)
        if self.__shadow is not None:
            if option.state & QtWidgets.QStyle.State_Selected:
                w = self.__boundingRect.width()*0.1//2
                painter.translate(-w, -w)
                painter.scale(1.1, 1.1)
            elif option.state & QtWidgets.QStyle.State_MouseOver:
                painter.translate(-1, -1)
            self.__shadow.setEnabled(self.isSelected())
        if self.__clip is not None:
            x, y,_,_ = self.__clip.getRect()
            painter.translate(-x, -y)
            painter.setClipRect(self.__clip)
        painter.drawPixmap(0, 0, self.pixmap)

    def shape(self):
        path = QtGui.QPainterPath()
        path.addRect(self.__boundingRect)
        return path

    def boundingRect(self):
        return self.__boundingRect

    def getRect(self):
        return self.__boundingRect.getRect()

    def mousePressEvent(self, event):
        super().mousePressEvent(event)
        self.checked.emit(event, getattr(self, 'spot', None))
        self.update()

    def mouseReleaseEvent(self, event):
        super().mouseReleaseEvent(event)
        self.update()

    def setEffect(self):
        shadow = QtWidgets.QGraphicsDropShadowEffect()
        shadow.setOffset(self.__boundingRect.topLeft())
        shadow.setColor(Qt.blue)
        shadow.setBlurRadius(self.__boundingRect.width())
        shadow.setEnabled(False)
        self.setGraphicsEffect(shadow)
        return shadow


class ButtonItem(QtWidgets.QGraphicsObject):
    trigger = Signal()

    def __init__(self, parent=None, **kwds):
        self._text = kwds.pop('text', '')
        self._font = kwds.pop('font', QtGui.QFont())
        self._rect = kwds.pop('rect', QtCore.QRectF())
        self._action = kwds.pop('action', None)
        super().__init__(parent, **kwds)
        self._fill = kwds.pop('fill', Qt.cyan)
        trigger = kwds.pop('trigger', None)
        if self._action:
            self.trigger.connect(self._action.trigger)
        if callable(trigger):
            self.trigger.connect(trigger)
        self.setAcceptHoverEvents(True)                              # 可悬停
        self.setCacheMode(self.ItemCoordinateCache)
        # self._fill = handy.readPixmap('image/button.png')

    def boundingRect(self):
        return self._rect

    def paint(self, painter, option, widget):
        painter.setRenderHint(QtGui.QPainter.Antialiasing)
        pen = Qt.black
        if option.state & QtWidgets.QStyle.State_Sunken:                       # 按下
            painter.translate(2, 2)
        elif option.state & QtWidgets.QStyle.State_MouseOver:                    # 进入控件
            pen = Qt.red
        painter.setFont(self._font)
        painter.setBrush(self._fill if self.isEnabled()
                         else QtGui.QColor('gainsboro'))
        painter.drawRoundedRect(self._rect.translated(2, 2), 15, 15)
        painter.setPen(pen)
        painter.drawText(self._rect, Qt.AlignCenter, self._text)
        # transform = QtGui.QTransform()
        # transform.shear(0.5, 0.7)
        # painter.translate(0,0)
        # painter.rotate(45)
        # painter.scale(1.5, 1.5)
        # painter.setTransform(transform)
        # painter.resetTransform()

    def mousePressEvent(self, event):
        self.trigger.emit()
        self.update()

    def mouseReleaseEvent(self, event):
        self.update()


class TextItem(QtWidgets.QGraphicsTextItem):
        # self.setTextInteractionFlags(Qt.TextEditable)
        # self.setTextInteractionFlags(Qt.TextEditorInteraction)
        # self.linkHovered .connect(lambda *h:print([h]))
        # self.setTextWidth(center+edge)
    def __init__(self, text, parent=None, **kwds):
        self.__font = kwds.pop('font',  QtGui.QFont())
        super().__init__(text, parent, **kwds)
        self.text = text
        self.setFont(self.__font)
        self.__pen = kwds.pop('pen', QtGui.QPen(Qt.NoPen))


class Touch(QtCore.QObject):
    TOUCHNAME = {
        QtCore.QEvent.TouchBegin: '触摸开始',
        QtCore.QEvent.TouchUpdate: '触摸更新',
        QtCore.QEvent.TouchEnd: '触摸结束', }
    dd, count, viewbox = {}, 0, None

    def __init__(self):
        super().__init__()

    @classmethod
    def getTouch(cls, event, cname=None):
        logs = [] if cname is not None else None
        touch = QtGui.QTouchEvent(event)
        points = touch.touchPoints()
        def detailed(ix, last, end):
            a = int(last.x()), int(last.y())
            b = int(end.x()), int(end.y())
            result = {'last':a, 'end':b}
            Touch.dd[f'触点{ix+1}'] = result
            if isinstance(logs, list):
                logs.append(result)
        cls.count = len(points)
        for ix,point in enumerate(points):
            last = point.lastPos()
            end = point.pos()
            # width = point.ellipseDiameters().width()
            detailed(ix, last, end)
        if cname is not None:
            log(cname, cls.TOUCHNAME.get(event.type(), 'Touch'), logs)
        return True

    @staticmethod
    def getpos(cname, name, event):
        p = event.pos()if hasattr(event, 'pos') else QtCore.QRect()
        pos = tuple(map(int,[p.x(),p.y()]))
        log(cname, name, pos)
        Touch.dd[name] = pos
        if name == "松开" and Touch.count:
            scene = getattr(Touch.viewbox, 'scene', None)
            p=getattr(scene(), 'elapsed', lambda:None)()

    @staticmethod
    def redraw(remove=True):
        if Touch.viewbox:
            Touch.viewbox.redraw()
        if not remove:return
        for k in Touch.dd.copy():
            if k.startswith('触点'):
                Touch.dd.pop(k)


class ChessScene(QtWidgets.QGraphicsScene):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.mousePressTime = QtCore.QElapsedTimer()
        self.mousePressTime.start()
        self.setFocusOnTouch(True) # 焦点放在触摸上
        self.setItemIndexMethod(self.NoIndex)

    def elapsed(self):
        elaps = self.mousePressTime.elapsed()  # 开始后时间
        Touch.dd["触摸时间"] = elaps
        return elaps
        self.mousePressTime.hasExpired(3000)  # 开始3秒内 True 否则 False

    def restart(self):
        self.mousePressTime.restart() # 计时清0
        Touch.dd.pop("触摸时间", 0)

    def keyPressEvent(self, event):
        if event.key() == Qt.Key_Escape:
            w.close()
        handy.master.exit_info()

    def event(self, event):
        etype = event.type()
        if etype == QtCore.QEvent.TouchBegin:
            self.restart()
        elif etype == QtCore.QEvent.TouchUpdate:
            0
        elif etype == QtCore.QEvent.TouchEnd:
            self.elapsed()
        return super().event(event)


class ViewBox(QtWidgets.QGraphicsWidget):
    def __init__(self, parent=None, **kwds):
        self.__pen = kwds.pop('pen', QtGui.QPen(Qt.NoPen))#
        self.__brush = kwds.pop('brush', QtGui.QBrush(Qt.NoBrush))
        self.rectf = kwds.pop('rect', QtCore.QRectF())
        super().__init__(parent, **kwds)
        self.addedItems = []
        self.background = QtWidgets.QGraphicsRectItem(self.rectf)
        self.background.setFlag(self.ItemStacksBehindParent) # 显示在最底层
        self.background.setPen(self.__pen)
        self.background.setBrush(self.__brush)
        self.background.setParentItem(self)

    def boundingRect(self):
        return self.background.boundingRect()

    def addItem(self, item):
        """ 项目添加到视图 """
        if item.zValue() < self.zValue():
            item.setZValue(self.zValue() + 1)
        scene = self.scene()
        if scene is not None and scene is not item.scene():
            scene.addItem(item)
        item.setParentItem(self)
        self.addedItems.append(item)

    def removeItem(self, item):
        """ 从此视图中删除项目 """
        item.setParentItem(None)
        scene = self.scene()
        if scene is not None:
            scene.removeItem(item)
        if item in self.addedItems:
            self.addedItems.remove(item)

    def redraw(self):
        for item in self.addedItems:
            item.update()

    def clear(self):
        for item in self.addedItems.copy():
            self.removeItem(item)


class CreateHead(ViewBox):
    def __init__(self, rect, head_pos=(), parent=None, **kwds):
        barh, edge = (lambda h:(h, h/6))(rect.height())
        rect = QtCore.QRectF(0, 0, barh-edge*3, barh-edge*2)
        super().__init__(parent, rect=rect)
        if not head_pos:
            kname, head_pos = self.get_name(), random.choice(handy.head_pos)
        else:
            kname = handy.config.get('myname')
            if not kname:
                kname = self.get_name()
                handy.config.set('myname', kname)
        kname = (lambda x:len(x)>5 and f'{x[:5]} ...' or x)(kname)
        (hx, hy), hw = head_pos, 40
        clip = QtCore.QRectF(hw*hx, hw*hy, hw, hw)
        self.head_item = PixmapItem('image/head.png', clip=clip, scale=rect.width()/hw)
        self.addItem(self.head_item)
        font = QtGui.QFont("Arial", int(edge/4), QtGui.QFont.Bold)
        self.text_item = TextItem(kname, pos=QtCore.QPointF(0, rect.width()), font=font)
        self.addItem(self.text_item)

    def get_name(self):
        fname = f"{handy.config_dir}/names.txt"
        if os.path.isfile(fname):
            try:
                with open(fname, 'r', encoding='utf-8') as buf:
                    nickname = buf.read().split('\n')
            except Exception:
                nickname = ["起名真难"]
        return random.choice(nickname)


class Progress(QtWidgets.QGraphicsObject):
    trigger = Signal()
    def __init__(self, parent=None, **kwds):
        self._rect = kwds.pop('rect', QtCore.QRectF())
        super().__init__(parent, **kwds)

    def boundingRect(self):
        return self._rect

    def paint(self, painter, option, widget):
        painter.setRenderHint(QtGui.QPainter.Antialiasing)
        if not self._cur:return
        painter = QtGui.QPainter(self)
        painter.setPen(QtCore.Qt.NoPen)
        painter.setRenderHints(QtGui.QPainter.Antialiasing | QtGui.QPainter.TextAntialiasing)
        painter.setBrush(self.back_color)
        painter.drawEllipse(self._edge)
        painter.setBrush(self.edge_color)
        painter.drawPie(self._edge, 90*16, (-360*16)//self._span*self._cur)
        painter.setBrush(self.core_color)
        painter.drawEllipse(self._core)
        painter.setPen(self.text_color)
        painter.drawText(self._core, QtCore.Qt.AlignCenter, "{:.0%}".format(self._cur/self._span))


class TopBar(ViewBox):
    def __init__(self, parent=None, **kwds):
        super().__init__(**kwds)
        self.edge = self.rectf.height()/6
        self.head_box = CreateHead(self.rectf)
        self.head_box.setPos(QtCore.QPointF(self.edge, self.edge))
        self.addItem(self.head_box)

        x,y,w,h = self.rectf.getRect()
        rect = QtCore.QRectF(0, 0, w/3*2, h//4)
        self.top_eat = BeEaten(rect=rect, pen=Qt.red, pos=QtCore.QPointF(w/3, h/4*3))
        self.addItem(self.top_eat)

        r = self.rectf.adjusted(self.edge*5, 10, -10, -20)
        parent.monitor.setPos(0, -self.edge*6)
        parent.monitor.setOpacity(0.8)
        parent.monitor.setRect(r)


class OperateBar(ViewBox):
    def __init__(self, parent=None, **kwds):
        super().__init__(**kwds)
        self.parent = parent
        self.menu = QtWidgets.QMenu()
        self.edge = self.rectf.height()/6
        self.my_head = list(map(int, handy.config.get('my_head')))
        self.head_box = CreateHead(self.rectf, self.my_head)
        self.addItem(self.head_box)
        px = self.rectf.width() - self.head_box.rectf.width()
        self.head_box.setPos(QtCore.QPointF(px-self.edge, self.edge))

        x,y,w,h = self.rectf.getRect()
        rect = QtCore.QRectF(0, 0, w/3*2, h//4)
        self.bot_eat = BeEaten(rect=rect, pen=Qt.red)
        self.addItem(self.bot_eat)

        self.button_box = ViewBox()
        self.addItem(self.button_box)

    def create_button(self, ):
        game = self.parent.chess_game
        option = [("翻转", game.reversal), ("认输", game._zmq.stop),
                  ("悔棋", game.last_step), ("提示", game.next_step), ]
        bx, by, bw, bh = self.rectf.getRect()
        font = QtGui.QFont("Arial", int(bh/20), QtGui.QFont.Bold)
        btw, bth = (bw-bh-20)//4, bh//4
        self.button_box.setGeometry(QtCore.QRectF(0, 0, bw-bh, bth))
        for i,(name,func) in enumerate(option):
            action = self.menu.addAction(name, func)
            rect = QtCore.QRectF(i*btw, 0, btw-10, bth)
            btn = ButtonItem(text=name, font=font, rect=rect, action=action)
            self.button_box.addItem(btn)
        self.button_box.setPos(bh/6, bh//2)

    def choose_head(self):
        vbox = ViewBox(rect=QtCore.QRectF(0,0,320,520), brush=Qt.white)
        self.parent._view.addItem(vbox)
        head_pos = handy.head_pos.copy()
        for y in range(13):
            for x in range(8):
                if not head_pos:break
                px, py = head_pos.pop(0)
                rect = QtCore.QRectF(px*40, py*40, 40, 40)
                pos = QtCore.QPointF(40*x, 40*y)
                pix = PixmapItem('image/head.png', vbox, clip=rect, pos=pos)
                pix.checked.connect(lambda e,s,i=(px, py),j=pix,v=vbox:self.head_clicked(i,j,v))
        vbox.setScale((self.parent.width()-100)/320)
        vbox.setPos(50, 50-self.rectf.height())

    def head_clicked(self, pos, obj, vbox):
        old = self.head_box.head_item
        obj.setPos(old.pos())
        obj.setScale(old.scale())
        self.head_box.removeItem(old)
        self.head_box.addItem(obj)
        self.head_box.head_item = obj
        handy.config.set('my_head', list(pos))
        self.parent._view.removeItem(vbox)


class ChessView(QtWidgets.QGraphicsView):
    def __init__(self, parent=None):
        super().__init__(parent)
        self.chess_state = 1
        self._scene = ChessScene()
        self.setScene(self._scene)
        self._view = ViewBox()
        self._scene.addItem(self._view)
        self._menu = QtWidgets.QMenuBar()

        self.setup_ui()
        self.setFrameStyle(self.NoFrame)
        self.setObjectName('chess-window')
        background = handy.readPixmap('image/background.png')
        self.setBackgroundBrush(QtGui.QBrush(background))
        self.setVerticalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOff)         # 去滚动条
        self.setHorizontalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOff)       # 去滚动条
        self.setRenderHints(QtGui.QPainter.HighQualityAntialiasing |
                QtGui.QPainter.Antialiasing | QtGui.QPainter.SmoothPixmapTransform)  # 抗锯齿

    def resizeEvent(self, event):
        self.fitInView(self._scene.sceneRect(), Qt.KeepAspectRatio)
        super().resizeEvent(event)

    def setup_ui(self):
        self._view.clear()
        self._menu.clear()

        self.monitor = LegendItem(info=True)
        self.monitor.setVisible(False)
        self._view.addItem(self.monitor)

        self.legend = LegendItem(self)
        self.legend.setVisible(False)
        self._view.addItem(self.legend)

        self.petyle = int(handy.config.get('petyle'))
        self.board = PixmapItem(handy.readPixmap(f'piece/{self.petyle}board.png', 'image'))
        self._view.addItem(self.board)

        bw, bh, barh = self.reScreen()
        rect, brush = QtCore.QRectF(0, 0, bw-4, barh), QtGui.QBrush(Qt.gray)
        self.topbar = TopBar(self, rect=rect, brush=brush, pos=QtCore.QPointF(2, -barh))
        self._view.addItem(self.topbar)
        self.operate = OperateBar(self, rect=rect, brush=brush, pos=QtCore.QPointF(2, bh))
        self._view.addItem(self.operate)

        self.chess_game = ChessGame(self)
        self._view.addItem(self.chess_game)

        self.operate.create_button()
        self.create_menu()
        self.chess_game.init_game()

    def reScreen(self):
        size = handy.getScreen().getRect()[2:]
        bw, bh = self.board.getRect()[2:]
        barh = int(bh / 3.2)
        if operator.gt(*size):  # 电脑宽大于高
            self.resize(*map(int, [bw, min(bh+barh, size[1]-100)]))
            handy.centerScreen(self)
        else:
            self.scale(size[0] / bw, size[0] / bw)
        return bw, bh, barh

    def create_menu(self):
        self.chessMenu = self._menu.addMenu("&切换")
        self.uncover_action = self.chessMenu.addAction("揭棋", lambda: self.switch_chess(1))
        self.uncover_action.setEnabled(self.chess_state != 1)
        self.chess_action = self.chessMenu.addAction("象棋", lambda: self.switch_chess(2))
        self.chess_action.setEnabled(self.chess_state == 1)
        self.monitor_action = QtWidgets.QAction("监视窗", checkable=True,
                            checked=False, toggled=self.monitor.setVisible)
        self.chessMenu.addAction(self.monitor_action)
        self._menu.addSeparator()

        self.functionMenu = self._menu.addMenu("&功能")
        self.cheat_action = QtWidgets.QAction("透视", self, enabled=True, triggered=self.create_cheat)
        for action in self.operate.menu.actions()+[self.cheat_action]:
            self.functionMenu.addAction(action)

        self.settingMenu = self._menu.addMenu("设置")
        self.ptyleMenu = self.settingMenu.addMenu("&棋子样式")
        options = ["高像素", "中像素", "低像素"]
        self.menu_group(self.ptyleMenu, options, 'petyle', 'self.setup_ui()')

        self.animaMenu = self.settingMenu.addMenu("&动画效果")
        self.curve = int(handy.config.get('curve'))
        options = ParallelMachine.animationEffects
        execs = 'self.chess_game.reversal()'
        self.menu_group(self.animaMenu, options, 'curve', execs)

        self._menu.addSeparator()
        self.headMenu = self.settingMenu.addAction("&我的头像", self.operate.choose_head)
        self.test_action = self._menu.addAction("测试", self.chess_game._test)

    def menu_group(self, menu, options, name, execs=None):
        def action_click(action, value):
            action.setChecked(True)
            menu.setDefaultAction(action)
            setattr(self, name, value)
            handy.config.set(name, value)
            if execs:exec(execs)

        group = QtWidgets.QActionGroup(self)
        default = int(handy.config.get(name))
        for i,j in enumerate(options):
            action = QtWidgets.QAction(j, group, checkable=True, enabled=True)
            action.triggered.connect(lambda e,a=action,v=i:action_click(a, v))
            if i == default:
                action.setChecked(True)
                menu.setDefaultAction(action)
            menu.addAction(action)

    def create_cheat(self):
        """ 偷窥棋子 """
        if self.chess_state != 1 or not self.chess_game.spot_back:
            return
        self.legend.setRect(self.board.boundingRect())
        self.legend.setOpacity(0.6)
        self.legend.picture = QtGui.QPicture()
        painter = QtGui.QPainter()
        painter.begin(self.legend.picture)
        for spot, back in self.chess_game.spot_back.items():
            if not back:continue
            pos = QtCore.QPointF(*self.chess_game.spot_pos[spot])
            painter.drawPixmap(pos, handy.readPixmap(f'piece/{self.petyle}{back}.png'))
        self.legend.update()

    def switch_chess(self, state):
        """ 切换 """
        if state == self.chess_state:
            return
        self.chess_state = state
        if state == 1:
            self.uncover_action.setEnabled(False)
            self.chess_action.setEnabled(True)
            self.cheat_action.setEnabled(True)
        else:
            self.uncover_action.setEnabled(True)
            self.chess_action.setEnabled(False)
            self.cheat_action.setEnabled(False)
        self.chess_game.init_game()

    def mouseReleaseEvent(self, event):
        self.monitor.update()
        super().mouseReleaseEvent(event)
        # event.accept()
        # event.ignore()

    def mouseMoveEvent(self, event):
        Touch.dd['坐标'] = event.x(), event.y()
        self.monitor.update()
        super().mouseMoveEvent(event)

    def mouseDoubleClickEvent(self, event):
        Touch.dd['双击'] = event.x(), event.y()
        super().mouseDoubleClickEvent(event)

    def mousePressEvent(self, event):
        super().mousePressEvent(event)
        Touch.dd['单击'] = event.x(), event.y()
        if self.legend.picture:
            del self.legend.picture

    def closeEvent(self, event):
        super().closeEvent(event)

    def exit_info(self):
        self.chess_game._zmq.stop()
        self.monitor.show()


class ZmqManage(QtCore.QObject):
    def __init__(self, parent=None):
        super().__init__()
        self.parent = parent
        self.server = None
        self.client = None
        self.redtime = 0
        self.blocktime = 0

    def send(self, *args):
        if self.client is None:
            self.start()
        self.client.send(args)

    def start(self, *args):
        self.parent.game_state = 1
        if self.server is None:
            self.server = Server()
            self.server.start()
        if self.client is None:
            self.client = Client()
            recv = getattr(self.parent, 'recv', None)
            if callable(recv):
                self.client.dataOut.connect(recv)
            self.client.start()
        # self.blocktime = QtCore.QElapsedTimer()
            # self.blocktime.start()
            # elaps = self.blocktime.elapsed()  # 开始后时间
            # self.blocktime.hasExpired(3000)  # 开始3秒内 True 否则 False
            # self.blocktime.restart() # 计时清0

    def stop(self):
        self.parent.game_state = 0
        if self.client is not None:
            self.client.send('stop')
            self.server.join()
            recv = getattr(self.parent, 'recv', None)
            if callable(recv):
                self.client.dataOut.disconnect(recv)
            self.server = None
            self.client = None


class BeEaten(ViewBox):
    def __init__(self, parent=None, **kwds):
        super().__init__(parent, **kwds)

    def add_piece(self, piece):
        """   """


class CursorItem(QtWidgets.QGraphicsItem):
    def __init__(self, parent=None, **kwds):
        super().__init__(parent, **kwds)
        self.parent = parent
        self._rect = parent.rectf
        self.start_pos = None
        self.end_pos = None
        self.offset = parent.offset
        self._rect = QtCore.QRectF()
        self.__background = Qt.white
        self.__w = self.offset[-1]/4

    def set_cursor(self, start, end):
        self.start_pos = start[0]+self.__w, start[1]+self.__w
        self.end_pos = end
        # log(self.offset, [start, end], self._rect)
        # self.start_center = [i+j/2 for i,j in zip(start, self.offset[2:])]
        # self.end_center = QtCore.QPointF(*[i+j/2 for i,j in zip(end, self.offset[2:])])
        # log(self.start_center)
        self.update()

    def boundingRect(self):
        return self._rect

    def paint(self, painter, option, widget):
        x, y, w, h = self.offset
        if self.start_pos:
            painter.setBrush(Qt.NoBrush)
            painter.setPen(QtGui.QPen(Qt.white, h/20))
            rect = QtCore.QRectF(*self.start_pos, h/2,h/2)
            painter.drawEllipse(rect)
            # painter.setBrush(Qt.white)
            # painter.setPen(Qt.NoPen)
            # painter.drawEllipse(*self.start_pos, 55,55)
        # painter.fillRect(self._rect, self.__background)

    def create_ellipse(self, diam, pos):
        """ 创建提示路线点实例 """
        # rect, radius = QtCore.QRectF(0, 0, diam, diam), diam/2
        ellipse = QtWidgets.QGraphicsEllipseItem()
        # shadow = QtWidgets.QGraphicsDropShadowEffect()
        # shadow.setOffset(self.__boundingRect.topLeft())offset, 
        # shadow.setColor(Qt.blue)
        # shadow.setBlurRadius(self.__boundingRect.width())
        # shadow.setEnabled(False)
        # ellipse.setGraphicsEffect(shadow)
            # self.create_ellipse()
            # self.clean.append(se)

        



class ChessGame(ViewBox):
    def __init__(self, parent=None, **kwds):
        super().__init__(**kwds)
        self.parent = parent
        self.game_state = 0
        self._zmq = ZmqManage(self)
        self.board = parent.board
        self.board.checked.connect(self.board_event)
        board_offset = handy.config.get(f'board_offset{parent.petyle}')
        self.offset = list(map(int, board_offset))
        self._sign = CursorItem(self)
        # self.addItem(self._sign)
        self.chi = handy.createMusic('./chi.mp3')
        self.luozhi = handy.createMusic('./luozhi.mp3')
        self.cannot = handy.createMusic('./cannot.mp3')
        self.jiangjun = handy.createMusic('./jiangjun.mp3')
        self.nine = [54, 55, 56, 70, 71, 72, 86, 87, 88,
                    166, 167, 168, 182, 183, 184, 198, 199, 200] # 九宫
        self.coordinates()

    def home_pos(self, piece):
        """ 棋子初始位置,  """
        return self.piece_home[not(piece & 32)]

    def getour(self):
        """ 返回对手在上,执红棋还是黑棋 16:帅, 32:将 """
        gp = lambda j:[self.spot_piece.get(i,0) for i in j]
        if 16 in gp(self.nine[9:]) or 32 in gp(self.nine[:9]):
            return 'black'
        elif 16 in gp(self.nine[:9]) or 32 in gp(self.nine[9:]):
            return 'red'

    def coordinates(self):
        """ 创建 16x16 棋盘位置坐标, """
        self.spot_pos = {}
        sx, sy, sw, sh = self.offset
        for y in range(3, 13):
            for x in range(3, 12):
                spot, pos = y*16+x, (sx + (x-3)*sw, sy + (y-3)*sh)
                self.spot_pos[spot] = pos
        self.piece_home = [QtCore.QPointF(sw, sy-sh*3),
                           QtCore.QPointF(sw*7, sh*11)]

    def pgn_surface(self, pgn=None):
        """ 读取PGN棋谱 [帅仕相馬車炮兵将士象馬車炮卒] """
        if pgn is None:
            pgn = 'rnbakabnr/9/1c5c1/p1p1p1p1p/9/9/P1P1P1P1P/1C5C1/9/RNBAKABNR w - - 0 1'.split()
        pieces = dict(zip('KABNRCPkabnrcp', (16, 17, 18, 19, 20, 21, 22, 32, 33, 34, 35, 36, 37, 38)))
        self.goto = pgn[1] != 'b'
        self.__step = (not self.goto and (int(pgn[-1]) + 1)) or int(pgn[-1])
        order = []
        for i in pgn[0]:                  # 棋子布局
            if i in pieces:
                order.append(pieces[i])
            elif i.isdigit():
                order.extend([0]*int(i))
        return OrderedDict([((i+3)*16+j+3, order[i*9+j]) for i in range(10) for j in range(9)])

    def random_opening(self, pgn=None):
        """ 创建揭棋随机开局 """
        red_piece = [17, 18, 19, 20, 21]*2 + [22]*5
        random.shuffle(red_piece)
        black_piece = [33, 34, 35, 36, 37]*2 + [38]*5
        random.shuffle(black_piece)               # 打乱顺序
        spot_back, marshal = OrderedDict(), (199, 55)        # 将帅位置
        for sp, pe in self.spot_piece.items():
            piece = black_piece if sp < 115 else red_piece
            spot_back[sp] = 0 if sp in marshal or not pe else piece.pop()
        return spot_back

    def create_point(self, parent, diam, color, pos):
        """ 创建提示路线点实例 """
        rect, radius = QtCore.QRectF(0, 0, diam, diam), diam/2
        ellipse = QtWidgets.QGraphicsEllipseItem(rect, parent)
        ellipse.setPen(QtGui.QPen(Qt.NoPen))
        ellipse.setPos(pos)
        radial = QtGui.QRadialGradient(radius, radius, radius)
        radial.setColorAt(0, color)
        radial.setColorAt(0.1, color)
        radial.setColorAt(1, Qt.transparent)
        ellipse.setBrush(radial)
        return ellipse

    def cannot_move(self, start, gope, end):
        """ 移动棋子错误动画 """
        self.cannot.play()
        old = self.find_spot(start)
        old_pos = QtCore.QPointF(*self.spot_pos[start])
        new_pos = QtCore.QPointF(*self.spot_pos[end])
        curve = QtCore.QEasingCurve.InOutBack
        goto = QtCore.QPropertyAnimation(old, b'pos', self)
        goto.setStartValue(old_pos)
        goto.setEndValue(new_pos)
        goto.setEasingCurve(curve)
        goto.setDuration(250)
        come = QtCore.QPropertyAnimation(old, b'pos', self)
        come.setStartValue(new_pos)
        come.setEndValue(old_pos)
        come.setEasingCurve(curve)
        come.setDuration(250)
        group = QtCore.QSequentialAnimationGroup(self)
        group.addAnimation(goto)
        group.addAnimation(come)
        group.start()

    def create_piece(self, spot_piece, reverse=False):
        """ 翻转棋子动画 """
        animation, count = [], 0
        for spot,piece in spot_piece.items():
            back = self.spot_back.get(spot)
            img_name = 'back' if back else piece
            item = self.find_spot(spot)
            if not img_name:continue
            if item is None and not reverse:
                # if reverse:
                    # log(spot,piece,back, [pix.spot for pix in self.addedItems])
                    # return 
                    # 
                # assert not reverse, (spot,piece, spot,piece)
                item = PixmapItem(f'piece/{self.parent.petyle}{img_name}.png', check=self.board_event)
                item.setPos(self.home_pos(piece)), self.addItem(item)
            elif item:
                spot = 254-spot if reverse else spot
            else :continue
            count, item.spot = count + 1, spot
            new_pos = QtCore.QPointF(*self.spot_pos[item.spot])
            animation.append([item, 'pos', item.pos(), new_pos, count*20+500])
        ParallelMachine(animation, curve=self.parent.curve, parent=self)

    def recv(self, value):
        if isinstance(value, (list, tuple)) and len(value)==3:
            self.move_piece(*value)
        else:
            log('接收', [value])

    def find_spot(self, spot):
        for pix in self.addedItems:
            if pix.spot == spot:
                return pix

    def pos_to_spot(self, pos):
        """ 位置坐标转网格 """
        ex, ey = pos.x(), pos.y()
        sx, sy, sw, sh = self.offset
        return int(((ey-sy)//sh+3)*16 + (ex-sx)//sw+3)

    def handle_residual(self, route=()):
        """ 提示路线处理 """
        if route:
            box = ViewBox()
            self.addItem(box)
            self.clean.append(box)
            sw = self.offset[2]/2.8
            for spot in route:
                pos = self.spot_pos[spot]
                point = QtCore.QPointF(pos[0]+sw, pos[1]+sw)
                self.create_point(box, sw, Qt.blue, point)
            return
        for i in self.clean.copy():           # 删除提示
            self.removeItem(self.clean.pop())

    def board_event(self, event, spot=None):
        """ 棋盘棋子点击事件 """
        self.handle_residual()
        if spot is None:                        # 棋盘事件
            spot = self.pos_to_spot(event.pos())
            if self.last_spot:
                args = self.last_spot + [spot]
                if spot in self.route:
                    self.move_piece(*args)
                    self.last_spot, self.route = [], []
                else:
                    self.cannot_move(*args)
            else:
                self.last_spot, self.route = [], []
        else:                                   # 棋子事件
            piece = self.spot_piece[spot]
            if self.last_spot and spot in self.route:
                args = self.last_spot + [spot]
                self.move_piece(*args)
            elif (self.goto and piece&16) or (not self.goto and piece&32):
                self.last_spot = [spot, piece]
                self.route = AllMoves.getRoute(spot, piece, self.spot_piece)
                self.handle_residual(self.route) # 添加路线
            else:
                self.last_spot, self.route = [], []

    def move_piece(self, start, piece, end):
        """ 移动棋子动画 """
        def finished():
            self.step = self.step + 1
            self.goto = not(self.goto)
            self.locus[self.step] = self.spot_piece
            # if not self.goto:
                # self._zmq.send(self.enemy, self.spot_piece)
            for obj in clear_obj:
                self.removeItem(obj)
            self.redraw(self.spot_piece)
            self.redraw(self.spot_back)

        animation, clear_obj, dark = [], [], None
        eat_dark = self.spot_back.get(end) # 被吃掉的暗子
        if eat_dark:
            self.spot_back[end] = 0
        eat_obj = self.find_spot(end)
        if eat_obj is not None: # 被吃掉的明子或反面对象
            clear_obj.append(eat_obj)
            self.chi.play()
        else: # 没有说明是空地
            self.luozhi.play()
        old_pos = QtCore.QPointF(*self.spot_pos[start])
        new_pos = QtCore.QPointF(*self.spot_pos[end])
        bright = self.find_spot(start) # 所要移动的棋子
        bright.spot = end
        # bright.setZValue(1)
        dark_piece = self.spot_back.get(start) # 移动的棋子是否是反面
        if dark_piece: # 如果是反面就要再创建翻开的实例对象
            clear_obj.append(bright) # 删除反面对象
            fname = f'piece/{self.parent.petyle}{dark_piece}.png'
            dark = PixmapItem(fname, pos=new_pos, check=self.board_event)
            dark.spot = end
            animation.append([dark, 'visible', False, True, 500]), self.addItem(dark)
            self.spot_back[start] = 0

            back = QtWidgets.QGraphicsRotation()
            back.setAxis(Qt.XAxis)
            bright.setTransformations([back]) # 用反面完成翻转动作
            back.angleChanged.connect(lambda:(lambda i:
                    (bright.setVisible(i<120), dark.setVisible(i>120)))(back.angle()))
            animation.append([back, 'angle', 0.0, 180.0, 500])
        animation.append([bright, 'pos', old_pos, new_pos, 500]) # 添加移动轨迹
        pm = ParallelMachine(animation, parent=self)
        pm.finished.connect(lambda:QtCore.QTimer.singleShot(500, finished))

        self.spot_piece[start] = 0
        self.spot_piece[end] = dark_piece or piece
        self.last_spot, self.route = [], []
        self._sign.set_cursor(*[self.spot_pos[i] for i in (start, end)])
        # if self.game_state == 0:
            # self._zmq.start()

    def init_game(self):
        self.clear()
        self.__step = 0                             # 几手
        self.goto = True                          # 哪方走棋
        self.clean = []                           # 随手清理对象
        self.route = []                           # 提示路线
        self.last_spot = []                       # 上次点击网格
        self.spot_back = {}                       # 揭棋背面棋子
        self.spot_piece = self.pgn_surface()      # 当前网格棋子局面
        self.locus = {-1: {}, 0: self.spot_piece} # 走棋轨迹列表
        if self.parent.chess_state == 1:
            self.spot_back = self.random_opening()
        self.create_piece(self.spot_piece)
        self.enemy = self.getour()

    @Property(int)
    def step(self):
        return self.__step

    @step.setter
    def step(self, value):
        self.__step = value

    def reversal(self):
        """ 棋盘翻转 """
        if self.last_spot:
            self.last_spot[0] = 254 - self.last_spot[0]
        self.spot_back = OrderedDict({254-i: j for i, j in self.spot_back.items()})
        pieces = OrderedDict({254-i: j for i, j in self.spot_piece.items()})
        self.redraw(pieces)
        self.create_piece(pieces, reverse=True)
        self.spot_piece = pieces
        if self.parent.legend.picture:
            self.parent.create_cheat()

    def set_enabled(self, name, check):
        for obj in self.function_button.get(name, []):
            obj.setEnabled(check)

    def last_step(self, ):
        check = self.step-1 in self.locus
        if check:
            self.create_piece(self.locus[self.step])
            # self.set_enabled("提示", check)front
        # elif len(self.locus) > 1:
            # self.set_enabled("悔棋", check)

    def next_step(self, ):
        check = self.step+1 in self.locus
        if check:
            self.step += 1
            self.create_piece(self.locus[self.step])

    def _test(self, ):
            self.redraw(self.spot_piece)
            self.redraw(self.spot_back)
    # import snoop
    # @snoop.snoop(depth=1)
    def redraw(self, data=None):
        if data is None:
            data = self.spot_piece
        for x, (i, j) in enumerate(data.items()):
            if not(x % 9):
                print()
            print('{:<4d}:{}'.format(i, j and '{:<4d}'.format(j) or '    '), end='   ')
        print('\n{}'.format('-'*99))


class MainWindow(QtWidgets.QMainWindow):
    def __init__(self, parent=None):
        super().__init__(parent)
        self.setAnimated(True)
        self.setWindowTitle("揭棋")
        self.setObjectName('main-window')
        self.setDockNestingEnabled(True)
        self.setAttribute(Qt.WA_AcceptTouchEvents, True)  # 接受触摸事件
        self.setWindowIcon(QtGui.QIcon(
            './resource/piece/016.png'))          # 设置图标
        QtCore.QTimer.singleShot(1000, lambda: self.setCursor(Qt.ArrowCursor))

        handy.master = chess = ChessView()
        menu = chess._menu
        helpMenu = menu.addMenu("&帮助")
        helpMenu.addAction("关于", self.about)
        helpMenu.addAction("关于 Qt", QtWidgets.QApplication.aboutQt)
        self.setMenuBar(menu)

        chess = self.createView(chess)
        self.setCentralWidget(chess)

    def createView(self, view):
        widget = QtWidgets.QWidget()
        layout = QtWidgets.QVBoxLayout(widget)
        layout.setContentsMargins(0, 0, 0, 0)
        layout.setSpacing(0)
        layout.addWidget(view)
        r = view.geometry().adjusted(4, 4, -4, -4)
        self.setGeometry(r)
        return widget

    def about(self):
        QtWidgets.QMessageBox.about(self, "关于",
                "作者: Obsession \n邮箱: 2608135678@qq.com")


def allattr(obj, ignore=(), exe=True):
        ignore = ['pyqtConfigure', 'dumpObjectInfo',
                  'isBlockedByModalPanel', ] + list(ignore)
        count, attrs = 0, dir(obj)

        def result(attr, func):
            try:
                return func()
            except:
                return f'{attr}(*, **)'
        for i, k in enumerate(attrs):
            if k in ignore or k.startswith('__') or k.endswith('__'):
                continue
            j = getattr(obj, k, '')
            res = result(k, j) if exe and callable(j) else j
            print('{:50}{}'.format(k, res))
        else:
            print('='*16, f"全部属性({i})", '='*16)
        print()




handy = Handy()
if __name__ == "__main__":
    app = QtWidgets.QApplication(sys.argv)
    w = MainWindow()
    w.show()
    sys.exit(app.exec_())



